#include <QtGui>
#include "paint.h"

#define y 5
#define x 11
#define E 1.0e-6

PainterPathWidget::PainterPathWidget(QWidget* pwgt) : QWidget(pwgt)
{
	
}

void PainterPathWidget::paintEvent(QPaintEvent*)
{
	QPainter painter(this);
	
painter.setRenderHint(QPainter::Antialiasing, true);
	QColor color;
	QString string;
	for (int i = 2; i < argc; ++i)
	{
		color.setRgb((255/argc)*(i-2), 0, 255-(255/argc)*(i-2));
		painter.setPen(QPen(color, 1.5, Qt::SolidLine, Qt::FlatCap, Qt::RoundJoin));
		qDebug() << "for i =" << i-1 << "argv =" << argv[i] << color;
		string = argv[i];
		
		FindPoint(string.toFloat());
		QPainterPath	path;
		path.moveTo(ptVector.first()*(width()/10));
		ptVector.pop_front();
		while (!ptVector.empty())
		{
			path.lineTo(ptVector.first()*(width()/10));
			ptVector.pop_front();
		}
		painter.setPen(QPen(color, 1.5, Qt::SolidLine, Qt::FlatCap, Qt::RoundJoin));
        	painter.drawPath(path);
		path.closeSubpath();
	}
	qDebug() << "Window size" << "w =" << width() << "h =" << height() ;
    }

void PainterPathWidget::Libman(float w)
{
	int counter_y, counter_x, iteration=0;
	float u_old[x+1][y+1], dx, dy, max;
	dx = 2.0/x;
	dy = 1.0/y;
	qDebug() << "W =" << w;
	/*Начальное заполнение матриц*/
	/*нули*/	
	for ( counter_x = 1; counter_x <= x; counter_x++)
		for (counter_y = 0; counter_y <= y; counter_y++)
	{
		u[counter_x][counter_y] = 0.0;
		u_old[counter_x][counter_y] = 0.0;
	}	
	/*значения по формуле*/
	for ( counter_y = 0; counter_y <= y; counter_y++)
	{
		if ((dy * counter_y) < 0.5)
		{
			u[0][counter_y] = 20 * dy * counter_y;
			u_old[0][counter_y] = 20 * dy * counter_y;
		}
		else
		{
			u[0][counter_y] = 20 * (1 - (dy * counter_y));
			u_old[0][counter_y] = 20 * (1 - (dy * counter_y));
		}
		
	}
	
	while (true)
	{
		max=0;
		for ( counter_y = 1; counter_y <= y-1; counter_y++)
		{	
			for (counter_x = 1; counter_x <= x-1; counter_x++)
			{
				/*сохраняем u от предыдущей итерации (он понадобится для ускоренного метода*/
				u_old[counter_x][counter_y] = u[counter_x][counter_y];
				/*находим u по обычному методу Либмана*/
				u[counter_x][counter_y]=(u[counter_x-1][counter_y]+u[counter_x+1][counter_y]+(dx*dx*(u[counter_x][counter_y-1]+u[counter_x][counter_y+1]))/(dy*dy))/(2+(2*dx*dx)/(dy*dy));
				/*модифицируем u по ускоренному методу Либмана*/
				u[counter_x][counter_y] = u[counter_x][counter_y]*w+(1-w)*u_old[counter_x][counter_y];
				/*находим максималное изменение*/
				if (max<qAbs(u[counter_x][counter_y]-u_old[counter_x][counter_y]))
				{
					max=qAbs(u[counter_x][counter_y]-u_old[counter_x][counter_y]);
				}
			}
		}
		++iteration;
		/*проверяем условие останова*/
		if (iteration >60000) 
		{
			qDebug() << "unstabl system";	
			break;
		}
		if (max < E) break;
	}
	/*Проверка максимального значения колличества итераций*/
	qDebug() << "iteration =" << iteration << "\t" << "max =" << max;
	
	for ( counter_x = 0; counter_x <= x; counter_x++)
	{
		for (counter_y = 0; counter_y <= y; counter_y++)
		{
			printf ("%'.4f\t",u[counter_x][counter_y]);
		}
		printf ("\n");
	}
}
/*Процедура поиска точек значения которых равны U_point, для нанесения изолинии*/
void PainterPathWidget::FindPoint(float U_point)
{
	float dU;
	int counter_y, counter_x;
	for ( counter_y = 0; counter_y <= y-1; counter_y++)
	{	
		for (counter_x = 0; counter_x <= x; counter_x++) 
		{
			/*проверяем есть ли наша точка между текущей и следующей позицией*/
			if ( ((u[counter_x][counter_y] >= U_point) & (u[counter_x][counter_y+1] <= U_point)) || ((u[counter_x][counter_y] <= U_point) & (u[counter_x][counter_y+1] >= U_point)))
			{
				dU=qAbs((u[counter_x][counter_y]-U_point)/(u[counter_x][counter_y]-u[counter_x][counter_y+1]));
				/*помещаем следующую точку в очередь*/
				ptVector.push_back(QPointF(counter_x, counter_y+dU));
			}
		}
	}
 	/*Сортируем точки по координате Y*/
	sortPoint();
}

/*Процедура сортировки точек*/
void PainterPathWidget::sortPoint()
{
	QPointF pt, ptBuffer;
	int i, j, count = ptVector.size();
	for ( i = count-1; i > 0; i--)
	{
		for (j = 0; j < i; j++)
		{
			if ( ptVector[j].ry() > ptVector[j+1].ry() )
			{
				ptBuffer = ptVector[j];
				ptVector[j] = ptVector[j+1];
				ptVector[j+1] = ptBuffer;
			}
		}
	}
			
}
